# Space-efficient Parallel Construction of Succinct Representations of Suffix Tree Topologies
Experiments for construction of succinct representations of suffix
tree topologies.

These are sources for the paper
	
	Space-efficient Parallel Construction of Succinct Representations of Suffix Tree Topologies

	by Uwe Baier, Timo Beller and Enno Ohlebusch (to appear in JEA 2016)

## About
This project deals with efficient sequential and parallel construction of suffix
tree topologies. We compare our new sequential and parallel algorithms against
existing ones. Existing approaches are described in:

- [A Simple Parallel Cartesian Tree Algorithm and its Application to Suffix Tree Construction](http://dl.acm.org/citation.cfm?doid=2632163.2661653)
  by Julian Shun and Guy E. Blelloch
- [Compressed suffix trees: design, construction, and applications](https://oparu.uni-ulm.de/xmlui/handle/123456789/3938)
  by Simon Gog

## What is contained
This bundle of files consist of the following parts:
1. The construction algorithms, contained in the `include`-directory
2. A benchmark, contained in the `src`-directory
3. A visualization for the benchmark data, contained in the `visualize`-directory
4. A set of test files (which need to be downloaded first, see below), contained
   in the `rcrdata`-directory

## Requirements
To run the benchmark, you need the following:
- a modern c++11 ready compiler such as [gcc](https://gcc.gnu.org/) version 4.7 or newer,
  supporting [OpenMP](http://openmp.org/wp/) with at least version 3.0 
  as well as [Cilk Plus](https://www.cilkplus.org/)
- the Succinct Data Structure Library [SDSL-LITE](https://github.com/simongog/sdsl-lite)
  version d533b2600950b4f878cb063ca0cd1bf340c53df4 or newer.

To visualize your results, the following programs are needed:
- [pdflatex], especially supporting pgf and pgfplotstable
- [awk]
- [rs](http://www.gsp.com/cgi-bin/man.cgi?topic=rs)

To download and set up the test files, the following programs are needed:
- [curl](https://curl.haxx.se/)
- [gzip]
- [twoBitToFa], a tool for converting 2-bit-files to fasta format. See instruction
  commands below.

## Installation
Most of the installation is straightforward, except for the following:

### SDSL
1. clone sdsl from the github repository https://github.com/simongog/sdsl-lite
2. install sdsl
3. set an environment variable onto the root of your sdsl-lite installation
   named SDSLLITE. Alternatively, go into the main Makefile of this project
   and replace the macro $(SDSLLITE) with the directory of your installation.

### twoBitToFa
1. Go to mirror http://hgdownload.soe.ucsc.edu/admin/exe/ and choose
   a directory with a compatible system
2. Download the file `twoBitToFa`, and store it into the `rcrdata`-directory
3. Make the file `twoBitToFa` executable, e.g. by

	chmod a+x twoBitToFa

Alternatively, you can download the kent source from UCSC and compile
and move the utility yourself if.

### Visualization
Go into the `visualize/exp` - directory and make both .sh scripts executable,
e.g. by the commands

	chmod a+x makefigures.sh
	chmod a+x maketables.sh

## Usage

### Benchmark
To run the benchmark, configure the file `benchmark.config` as you
require it, an example is already listed. Afterwards, call

	make benchmark

This should compile the benchmark, and placing .dat - files into the
results directory. Each file contains information about algorithm, used
number of processors and time needed to construct the representation,
given in milliseconds.

### Visualization
To visualize your data, call

	make visualize

If benchmark data is not available yet, the benchmark will be executed 
before the data gets visualized. After visualization, the file 
`visualize/bpsconstruct.pdf` contains the data visualized.

### Getting Test Data
To download the Test Data, go into the `rcrdata`-directory and call
	
	make

The data should be downloaded then. Don't forget to prepare the benchmark
before using the data.

### Replicating Computational Results
To replicate the computational results from our paper, just call

	make rcr

This command will automatically download the test data, sets up the benchmark
properly (Warning: the benchmark.config - file will be overwritten) and visualize
the data. The resulting pdf can be fetched from the `visualize` - directory.
