#!/bin/bash
# script expects a couple of resulting data files from the bps benchmark and
# creates latex code which displays a figure showing increase of speed.
# For each input file INPUT.dat, three files are generated:
# - INPUT-dfs-fig.tex contains data for bps dfs construction
# - INPUT-sct-fig.tex contains data for bps sct construction
# - INPUT-fig.tex contains data for both constructions

#write results to a tex file
#$1: file to append data to
#$2: name of testcase
#$3: filesize of file
#$4: sigma of file
#$5: first column of data field as row (e.g. processors	1	2	3)
#$6: transposed data field, excluding first column (see $5)
write_results(){
	fsMB=$(echo "scale=0;$3 / (1024*1024)" | bc)
	algcount=$(echo "$6" | wc -l)
	datafield="$5\n"	#initial data field
	for (( l=1; l<=$algcount; l++ )) 
	do					#remove zero entries
		line=$(echo "$6" | head -$l | tail -1)
		timings=$(echo "$line" | tr -d '[:alpha:]')	#and timings list
		line=$(echo "$line" | tr -c -d '[:alpha:]')	#extract algorithm
		for time in $timings ; do
			if [ $time -eq 0 ] ; then	#replace zero timings
				time=1
			fi
			speed=$(echo "scale=2;((($3 / $time) / 1024) * 1000) / 1024" | bc)
			line+="\t$speed"
		done
		datafield+="$line\n"
	done
	#retranspose data field
	datafield=$(echo -e -n "$datafield" | rs -c -C -T)
	#write inline pgf table for the data
	echo "\pgfplotstableread{" > $1
	echo "$datafield" >> $1
	echo "}\datatable" >> $1
	echo "\begin{tikzpicture}[use background]" >> $1
	echo "\begin{loglogaxis}[title={\(\mathsf{$2}\) (\(\sigma = $4\), \(n = $fsMB \) MB)}," >> $1
	echo "	xlabel=processors,ylabel={speed in \(\sfrac{\text{MB}}{\text{s}}\)}," >> $1
	echo "	xtick=data,log ticks with fixed point,log basis y=2," >> $1
	echo "	ymajorgrids=true, xmajorgrids=true]" >> $1
	algorithms=$(echo "$datafield" | head -1 | cut -f2- ) #extract algorithms and remove first entry
	for alg in $algorithms ; do
		echo "		\addplot table[x=processors,y=$alg] {\datatable};" >> $1
		echo "		\addlegendentry{\\$alg};" >> $1
	done
	echo "\end{loglogaxis}" >> $1
	echo "\end{tikzpicture}" >> $1	
}

for rfile in $@
do
	filesize=$(head -1 $rfile | tr -d -c '[:digit:]')  #fetch file size
	sigma=$(head -2 $rfile | tail -1 | tr -d -c '[:digit:]') #fetch sigma
	dt=$(tail -n +3 $rfile | rs -c -C -T)		#fetch data field and transpose it
	dtfirstcolumn=$(echo -e -n "$dt" | head -1)	#extract first row
	dt=$(echo -e -n "$dt" |  tail -n +2)		#remove first row from data field
	dtdfs=$(echo "$dt" | awk '/^bpsdfs|mwsctshun/ { print $0 }') #extract sada fields
	dtsct=$(echo "$dt" | awk '/^bpssct|mwsctshun/ { print $0 }') #extract sct fields
	tcname=$(basename -s .dat $rfile)
	dfsfigfile="$tcname-dfs-fig.tex"
	sctfigfile="$tcname-sct-fig.tex"
	figfile="$tcname-fig.tex"
	write_results $dfsfigfile $tcname $filesize $sigma "$dtfirstcolumn" "$dtdfs"
	write_results $sctfigfile $tcname $filesize $sigma "$dtfirstcolumn" "$dtsct"
	write_results $figfile $tcname $filesize $sigma "$dtfirstcolumn" "$dt"
done
