#!/bin/bash
# script expects a couple of resulting data files from the bps benchmark and
# creates latex code which displays a table showing construction speeds.
# Overall, three files are generated:
# - result-dfs-tbl.tex contains data for bps dfs construction
# - result-sct-tbl.tex contains data for bps sct construction
# - result-tbl.tex contains data for both constructions

dfstable="result-dfs-tbl.tex"
scttable="result-sct-tbl.tex"
table="result-tbl.tex"

#write results to a tex file
#$1: file to append data to
#$2: name of data file
#$3: filesize of file
#$4: sigma of file
#$5: transposed data field
write_results(){
	tcname=$(basename -s .dat $2)
	fsMB=$(echo "scale=0;$3 / (1024*1024)" | bc)
	algcount=$(echo "$5" | wc -l)
	echo "\hline\multirow{$algcount}{*}{ \begin{tabular}{c} \(\mathsf{$tcname}\) \\\\ \(\sigma = \) $sigma \\\\ \(n = \) $fsMB MB \end{tabular} }" >> $1
	for (( l=1; l<=$algcount; l++ ))
	do
		line=$(echo "$5" | head -$l | tail -1)
		alg=$(echo "$line" | tr -c -d '[:alpha:]')	#extract algorithm
		timings=$(echo "$line" | tr -d '[:alpha:]')	#and timings list
		echo -n " & \\$alg" >> $1
		for time in $timings
		do
			if [ $time -eq 0 ] ; then	#replace zero timings
				time=1
			fi		#compute speed in MB/s
			speed=$(echo "scale=2;((($3 / $time) / 1024) * 1000) / 1024" | bc)
			echo -n " & $speed " >> $1
		done
		echo " \\\\" >> $1
	done
}

#generate first two columns of table from first file
l1="\\begin{longtable}{cl"
l2="File & Algorithm"
if [ $# -ne 0 ] ; then
	usedprocs=$(tail -n +4 $1 | awk '{print $1}')
	upsize=$(echo $usedprocs | wc -w)
	l1+="*{$upsize}{r}"
	for pc in $usedprocs
	do
		l2+=" & \(n / P_{$pc}\)"
	done
fi
l1+="}"
l2+=" \\\\"

echo "$l1" > $dfstable
echo "$l2" >> $dfstable
echo "$l1" > $scttable
echo "$l2" >> $scttable
echo "$l1" > $table
echo "$l2" >> $table
for rfile in $@
do
	filesize=$(head -1 $rfile | tr -d -c '[:digit:]')  #fetch file size
	sigma=$(head -2 $rfile | tail -1 | tr -d -c '[:digit:]') #and sigma
	dt=$(tail -n +3 $rfile | rs -c -C -T | tail -n +2) #transpose data field
	dtdfs=$(echo "$dt" | awk '/^bpsdfs|mwsctshun/ { print $0 }') #extract dfs fields
	dtsct=$(echo "$dt" | awk '/^bpssct|mwsctshun/ { print $0 }') #extract sct fields
	echo "\\hline" >> $dfstable
	write_results $dfstable $rfile $filesize $sigma "$dtdfs"
	write_results $scttable $rfile $filesize $sigma "$dtsct"
	write_results $table $rfile $filesize $sigma "$dt"
done
le="\\end{longtable}"
echo "$le" >> $dfstable
echo "$le" >> $scttable
echo "$le" >> $table
